/*
  Code Checker(4).c
  Writes to the screen 
  No signals to the motors
*/

#include "simpletools.h" 
const int DO = 22, CLK = 23, DI = 24, CS = 25;

#define BufferCount 4
#define TRUE 1
#define FALSE 0
#define OutputStackSize (40 + 32)

int i;
int Xdir;
int Xstep;
int Ydir;
int Ystep;

char Buffer[BufferCount][512];
volatile char volatile BufferFull[BufferCount];
volatile char StopSending = FALSE;
volatile int BuffersRead = 0;
static int OutputStack[OutputStackSize];

void OutputDataThread(void *par);

int main()                                    // Main function
{
  print("Code Checker(2)\n\n");                            // Display test message
  Xdir=1;
  Xstep=1;
  Ydir=1;
  Ystep=1;
  
  sd_mount(DO, CLK, DI, CS);
  
  FILE * fp = fopen("plasma.bin", "wb" );
  
  for(i=0; i<4000000; i++)
  {  
  char byteToOutput = 0;
  if(i & 1)
  {
    byteToOutput = (1<<1) | (1<<3);
  }
  
  Buffer[0][i & 511] = byteToOutput;
  
  if((i & 511) == 511)
  {
    fwrite( Buffer[0],1,512,fp);
  }
  if((i & 16383) == 16383)
  {
    printi("writing...%d\r",i);
  }
}  
//  fflush(fp);
  fclose(fp);  
  
  //Start Jason's program

  printi("Begin send.\n"); 

  for(int i=0; i<BufferCount; i++)
  {
    BufferFull[i]=FALSE;
  }
  
  StopSending=FALSE;
  fp = fopen("plasma.bin","rb");
  
  int bufferToRead = 0;
  int readResult = 0;
  
  BuffersRead = 0;
     
  cogstart(&OutputDataThread, 0, OutputStack, sizeof(OutputStack) );  
  
  while(1)
  {
    while( BufferFull[bufferToRead] == TRUE )
    {
      ;
    }
    
    putChar('0' + bufferToRead);
    
    readResult = fread(Buffer[bufferToRead], 1, 512, fp);
    
    if( readResult > 0)
    {
      BuffersRead++;
      
      if(readResult < 512 )
      {
        memset( Buffer[bufferToRead] + readResult, 0, 512 - readResult );
        
        putChar( 'X' );
      }      
      BufferFull[ bufferToRead ] = TRUE;      
      bufferToRead = (bufferToRead + 1) % BufferCount;      
     }           
     else     
    { 
    putChar( 'E' );     
    break;      
    }
  }
  
  StopSending = TRUE;  
  fclose(fp);
  
  while(TRUE)
  {
    ;
  }
  low(0);//Stop the Cut
  print("Stopping the Plasma Cutter.\n");
  print("End of Cut.\n\n");                   
}

void OutputDataThread(void *par)
{
  int OutputRate = 53000;  //CutRate
  int OutputDelay = _clkfreq / OutputRate;  
  int bufferToSend = 0; 
  int buffersSent = 0;
   
  const int PinMask = (1<<0) | ( 1<<1) | ( 1<<2) | (1<<3);  
  const int DataShift = 0;  
  DIRA |= PinMask;  
  DIRA |= (1<<26);  
  int clockTick = CNT;
    
  while( (StopSending == FALSE) | (buffersSent < BuffersRead) )
  {
    while( BufferFull[bufferToSend] == FALSE )
    {
      clockTick = CNT;
    }
        
    for( int i=0; i<512; i++ )
    {
      int data = Buffer[bufferToSend][i] & 0x0F;      
      clockTick += OutputDelay;     
      waitcnt( clockTick );      
      OUTA = (OUTA & ~PinMask) | (data << DataShift);
    }
    buffersSent++;
    
    if( bufferToSend == 0 )
    {
      OUTA = OUTA ^ (1<<26);
    }    
    BufferFull[bufferToSend] = FALSE;    
    bufferToSend = (bufferToSend+1) % BufferCount;
  }
  OUTA = 0;
}  
